@@ -1,4 +1,4 @@
-# Copyright (c) 2003-4 Timothy Appnel
+# Copyright (c) 2003-2004 Timothy Appnel
 # http://www.timaoutloud.org/
 # This code is released under the Artistic License.
 #
@@ -49,7 +49,8 @@ my %xpath_prefix = (
 	thr=>"http://purl.org/rss/1.0/modules/threading/",
 	trackback=>"http://madskills.com/public/xml/rss/module/trackback/",
 	wiki=>"http://purl.org/rss/1.0/modules/wiki/",
-	xhtml=>"http://www.w3.org/1999/xhtml/"
+	xhtml=>"http://www.w3.org/1999/xhtml/",
+    xml=>"http://www.w3.org/XML/1998/namespace/"
 );
 my %xpath_ns = reverse %xpath_prefix;
 
@@ -62,6 +63,7 @@ sub new {
 	return $self;
 }
 
+# fix: use non-recursive means.
 sub root { $_[0]->parent ? $_[0]->parent->root : $_[0] ; }
 sub parent { $_[0]->{parent} = $_[1] if $_[1]; $_[0]->{parent}; }
 sub name { $_[0]->{name} = $_[1] if $_[1]; $_[0]->{name}; }
@@ -84,34 +86,28 @@ sub children {
 				$self->{__children}->{$tag}->[0];
 	} else {
 		return $self->{child_stack} ? 
-			@{ $self->{child_stack} } :
-				undef;
+			@{ $self->{child_stack} } : ();
 	}
 }
 
-sub children_names { keys %{ $_[0]->{__children} }; }
+sub children_names { keys %{ $_[0]->{__children} } || (); }
 sub attribute { $_[0]->{attributes}->{ $_[1] } = $_[2] if $_[2]; $_[0]->{attributes}->{ $_[1] } } 
 sub attributes { $_[0]->{attributes}=$_[1] if $_[1]; $_[0]->{attributes}; };
 
 *query = \&match;
 
 sub _xpath_name {
-	my $name = $_[1] || $_[0]->{name}; # for reuse.
-	my $ns;
-	($ns,$name) = $name =~m!^(.*?)([^/#]+)$!;
-	my $prefix =  $xpath_prefix{$ns} || '';
-	$prefix ? "$prefix:$name" : $name;
- }
- 
+    my $in = ref($_[0]) ? $_[0]->{name} : $_[0] ;
+    my($ns,$name) = $in =~m!^(.*?)([^/#]+)$!;
+    my $prefix =  $xpath_ns{$ns} || '';
+    # doubtful that this is how an undefined xpath namespace 
+    # should be handled.
+    $prefix && $prefix ne '#default' ? "$prefix:$name" : $name;
+}
+
 sub _xpath_attribute_names { 
-	my $self = shift;
-	return unless $self->{attributes};
-    my @a;
-    foreach (keys %{ $self->{attributes} }) {
-            my($ns,$name) = ($_=~ m!(.+?)(\w+)$!); 
-            push @a, join(':',$xpath_ns{$ns},$name);
-    }
-    @a;
+	return () unless $_[0]->{attr};
+    map { _xpath_name($_) } keys %{ $_[0]->{attr} };
 }
 
 sub _xpath_attribute {
@@ -125,122 +121,128 @@ sub _xpath_attribute {
 	} else {
 	    ($ns = $self->name)=~ s/\w+$//;
 	}
-	$self->attribute("$ns$name");
+	$self->{attr}->{"$ns$name"};
 }
 
 1;
 
 __END__
 
+=begin
+
 =head1 NAME
 
-XML::RSS::Parser::Element -- a simple object that holds one node in the XML::RSS::Parser parse tree.
+XML::RSS::Parser::Element -- a simple object that holds one node in
+the XML::RSS::Parser parse tree.
 
 =head1 DESCRIPTION
 
-XML::RSS::Parser::Element is a simple object that holds one node in the parse tree. Roughly based on
-L<XML::SimpleObject>.
+XML::RSS::Parser::Element is a simple object that holds one node in
+the parse tree. Roughly based on L<XML::SimpleObject>.
 
 =head1 METHODS
 
 =item XML::RSS::Parser::Element->new( [\%init] )
 
-Constructor for XML::RSS::Parser::Element. Optionally the name, value, attributes, root, and parent 
-can be set with a HASH reference using keys of the same name. See their associated functions below 
-for more.
+Constructor for XML::RSS::Parser::Element. Optionally the name,
+value, attributes, root, and parent can be set with a HASH
+reference using keys of the same name. See their associated
+functions below for more.
 
 =item $element->root( [$feed] )
 
-Returns a reference to the root element of the parse tree. A L<XML::RSS::Parser::Feed> 
-can be passed to optionally set the root element. The default is undefined.
+Returns a reference to the root element of the parse tree. A
+L<XML::RSS::Parser::Feed> can be passed to optionally set the root
+element. The default is undefined.
 
 =item $element->parent( [$element] )
 
-Returns a reference to the parent element. A L<XML::RSS::Parser::Element> object 
-or one of its subclasses can be passed to optionally set the parent. The 
-default is undefined.
+Returns a reference to the parent element. A
+L<XML::RSS::Parser::Element> object or one of its subclasses can be
+passed to optionally set the parent. The default is undefined.
 
 =item $element->name( [$extended_name] )
 
-Returns the name of the element as a SCALAR. This should by the fully namespace 
-qualified (extended)  name of the element and not the QName or local part. The 
-default is undefined. 
+Returns the name of the element as a SCALAR. This should by the
+fully namespace qualified (extended)  name of the element and not
+the QName or local part. The default is undefined.
 
 =item $element->value( [$value] )
 
-Returns a reference to the value (text contents) of the element. If an optional 
-SCALAR parameter is passed in the value (text contents) is set. Returns a null 
-string if not set.
+Returns a reference to the value (text contents) of the element. If
+an optional SCALAR parameter is passed in the value (text contents)
+is set. Returns a null string if not set.
 
 =item $element->append_value( $value )
 
-Appends the value of the SCALAR parameter to the object's current value. A 
-convenience method that is particularly helpful when working in 
-L<XML::Parser> handlers.
+Appends the value of the SCALAR parameter to the object's current
+value. A convenience method that is particularly helpful when
+working in L<XML::Parser> handlers.
 
 =item $element->attribute($name [, $value] )
 
-Returns the value of an attribute specified by C<$name> as a SCALAR. If 
-an optional second text parameter C<$value> is passed in the attribute is set.
+Returns the value of an attribute specified by C<$name> as a
+SCALAR. If an optional second text parameter C<$value> is passed in
+the attribute is set. Returns C<undef> if the attribute does not
+exist.
 
 =item $element->attributes( [\%attributes] )
 
-Returns a HASH reference contain attributes and their values as key value pairs. 
-An optional parameter of a HASH reference can be passed in to set multiple 
-attributes. NOTE: When setting attributes with this method, all existing 
-attributes are overwritten irregardless of whether they are present in the
-hash being passed in.
+Returns a HASH reference contain attributes and their values as key
+value pairs. An optional parameter of a HASH reference can be
+passed in to set multiple attributes. Returns C<undef> if no
+attributes exist. NOTE: When setting attributes with this method,
+all existing attributes are overwritten irregardless of whether
+they are present in the hash being passed in.
 
 =item $element->child( [$extended_name] )
 
-Constructs and returns a new element object making the current object 
-as its parent. An optional parameter representing the name of the new 
-element object can be passed. This should be the fully namespace 
-qualified (extended) name and not the QName or local part.
+Constructs and returns a new element object making the current
+object as its parent. An optional parameter representing the name
+of the new element object can be passed. This should be the fully
+namespace qualified (extended) name and not the QName or local
+part. Returns C<undef> if the child is not present.
 
 =item $element->children( [$extended_name] )
 
-Returns any array of child elements to the object. An optional parameter 
-can be passed in to return element(s) with a specific name. If called in 
-a SCALAR context it will return only the first element with this name. 
-If called in an ARRAY context the function returns all elements with 
-this name. If no elements exist as a child of the object, and undefined 
-value is returned.
+Returns any array of child elements to the object. An optional
+parameter can be passed in to return element(s) with a specific
+name. If called in a SCALAR context it will return only the first
+element with this name. If called in an ARRAY context the function
+returns all elements with this name. If no elements exist as a
+child of the object, and undefined value is returned.
 
 =item $element->children_names
 
-Returns an array contain the names of the objects children.
+Returns an array containing the names of the objects children.
+Empty if no children are present.
 
 =head2 XPath-esque Methods
 
 =item $element->query($xpath)
 
-Finds matching nodes using an XPath-esque query from anywhere in the 
-tree. See the L<Class::XPath> documentation for more information.
+Finds matching nodes using an XPath-esque query from anywhere in
+the tree. See the L<Class::XPath> documentation for more
+information.
 
 =item $element->match($xpath)
 
-Alias for the C<query> method. For compatability. C<query> is preferred.
+Alias for the C<query> method. For compatability. C<query> is
+preferred.
 
 =item $element->xpath
 
-Returns a unique XPath string to the current node which can be used 
+Returns a unique XPath string to the current node which can be used
 as an identifier.
 
 =head1 SEE ALSO
 
 L<XML::RSS::Parser>, L<XML::SimpleObject>, L<Class::XPath>
 
-=head1 LICENSE
-
-The software is released under the Artistic License. The terms of 
-the Artistic License are described at 
-L<http://www.perl.com/language/misc/Artistic.html>.
-
 =head1 AUTHOR & COPYRIGHT
 
-Except where otherwise noted, XML::RSS::Parser::Element is Copyright 
-2003-4, Timothy Appnel, cpan@timaoutloud.org. All rights reserved.
+Please see the XML::RSS::Parser manpage for author, copyright, and
+license information.
 
 =cut
 
